@@ -76,17 +76,13 @@ static gboolean
 set_gerror_from_pcre_error(int errcode,
                            GError **error)
 {
-        if (errcode < 0) {
-                PCRE2_UCHAR8 buf[128];
-                int n;
+        PCRE2_UCHAR8 buf[128];
+        int n;
 
-                n = pcre2_get_error_message_8(errcode, buf, sizeof (buf));
-                g_assert(n >= 0);
-                g_set_error_literal(error, VTE_REGEX_ERROR, errcode, (const char*)buf);
-                return FALSE;
-        }
-
-        return TRUE;
+        n = pcre2_get_error_message_8(errcode, buf, sizeof (buf));
+        g_assert(n >= 0);
+        g_set_error_literal(error, VTE_REGEX_ERROR, errcode, (const char*)buf);
+        return FALSE;
 }
 
 #else
@@ -191,13 +187,14 @@ vte_regex_new(const char *pattern,
                                (uint32_t)flags |
                                PCRE2_UTF |
                                (flags & PCRE2_UTF ? PCRE2_NO_UTF_CHECK : 0) |
-                               PCRE2_NEVER_BACKSLASH_C,
+                               PCRE2_NEVER_BACKSLASH_C |
+                               PCRE2_USE_OFFSET_LIMIT,
                                &errcode, &erroffset,
                                NULL);
 
-        if (code == 0) {
+        if (code == nullptr) {
                 set_gerror_from_pcre_error(errcode, error);
-                g_prefix_error(error, "Failed to compile pattern to regex at %" G_GSIZE_FORMAT ":",
+                g_prefix_error(error, "Failed to compile pattern to regex at offset %" G_GSIZE_FORMAT ":",
                                erroffset);
                 return NULL;
         }
@@ -282,8 +279,10 @@ vte_regex_jit(VteRegex *regex,
         g_return_val_if_fail(regex != NULL, FALSE);
 
         r = pcre2_jit_compile_8(regex->code, flags);
+        if (r < 0)
+                return set_gerror_from_pcre_error(r, error);
 
-        return set_gerror_from_pcre_error(r, error);
+        return TRUE;
 #else
         return set_unsupported_error(error);
 #endif /* WITH_PCRE2 */
