@@ -75,6 +75,8 @@
  * Design discussions: https://bugzilla.gnome.org/show_bug.cgi?id=738601
  */
 
+#include <glib.h>
+#include "config.h"
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -89,6 +91,8 @@
 
 #include "vteutils.h"
 
+G_BEGIN_DECLS
+
 #ifdef WITH_GNUTLS
 /* Currently the code requires that a stream cipher (e.g. GCM) is used
  * which can encrypt any amount of data without need for padding. */
@@ -621,22 +625,26 @@ _vte_snake_class_init (VteSnakeClass *klass)
  * - T..64k (T..10): Area not written to the file, most of that leaving sparse FS blocks (dots for unit testing)
  */
 
+#if !defined VTESTREAM_MAIN && defined WITH_GNUTLS
+        /* The IV (nonce) consists of the offset within the stream, and an overwrite counter so that
+         * we don't reuse the same IVs when a block at a certain logical offset is overwritten.
+         * The padding is there to make sure the structure is at least VTE_CIPHER_IV_SIZE bytes large.
+         * Assertion is made later that the real data fits in its first VTE_CIPHER_IV_SIZE bytes.
+         */
+        typedef struct _VteIv {
+                gsize offset;
+                guint32 overwrite_counter;
+                unsigned char padding[VTE_CIPHER_IV_SIZE];
+        } VteIv;
+#endif
+
 typedef struct _VteBoa {
         VteSnake parent;
         gsize tail, head;
 
 #if !defined VTESTREAM_MAIN && defined WITH_GNUTLS
         gnutls_cipher_hd_t cipher_hd;
-        /* The IV (nonce) consists of the offset within the stream, and an overwrite counter so that
-         * we don't reuse the same IVs when a block at a certain logical offset is overwritten.
-         * The padding is there to make sure the structure is at least VTE_CIPHER_IV_SIZE bytes large.
-         * Assertion is made later that the real data fits in its first VTE_CIPHER_IV_SIZE bytes.
-         */
-        struct _VteIv {
-                gsize offset;
-                guint32 overwrite_counter;
-                unsigned char padding[VTE_CIPHER_IV_SIZE];
-        } iv;
+        VteIv iv;
 #endif
         int compressBound;
 } VteBoa;
@@ -875,25 +883,24 @@ static gboolean
 _vte_boa_read_with_overwrite_counter (VteBoa *boa, gsize offset, char *data, _vte_overwrite_counter_t *overwrite_counter)
 {
         _vte_block_datalength_t compressed_len;
-        gboolean ret = FALSE;
-        char *buf = g_malloc(VTE_SNAKE_BLOCKSIZE);
+        char *buf = g_newa(char, VTE_SNAKE_BLOCKSIZE);
 
         g_assert_cmpuint (offset % VTE_BOA_BLOCKSIZE, ==, 0);
 
         /* Read */
         if (G_UNLIKELY (!_vte_snake_read (&boa->parent, OFFSET_BOA_TO_SNAKE(offset), buf)))
-                goto out;
+                return FALSE;
 
         compressed_len = *((_vte_block_datalength_t *) buf);
         *overwrite_counter = *((_vte_overwrite_counter_t *) (buf + VTE_BLOCK_DATALENGTH_SIZE));
 
         /* We could have read an empty block due to a previous disk full. Treat that as an error too. Perform other sanity checks. */
         if (G_UNLIKELY (compressed_len <= 0 || compressed_len > VTE_BOA_BLOCKSIZE || *overwrite_counter <= 0))
-                goto out;
+                return FALSE;
 
         /* Decrypt, bail out on tag mismatch */
         if (G_UNLIKELY (!_vte_boa_decrypt (boa, offset, *overwrite_counter, buf + VTE_BLOCK_DATALENGTH_SIZE + VTE_OVERWRITE_COUNTER_SIZE, compressed_len)))
-                goto out;
+                return FALSE;
 
         /* Uncompress, or copy if wasn't compressable */
         if (G_LIKELY (data != NULL)) {
@@ -905,11 +912,7 @@ _vte_boa_read_with_overwrite_counter (VteBoa *boa, gsize offset, char *data, _vt
                         g_assert_cmpuint (uncompressed_len, ==, VTE_BOA_BLOCKSIZE);
                 }
         }
-        ret = TRUE;
-
-out:
-        g_free(buf);
-        return ret;
+        return TRUE;
 }
 
 static gboolean
@@ -935,8 +938,8 @@ _vte_boa_write (VteBoa *boa, gsize offset, const char *data)
 
         /* The helper buffer should be large enough to contain a whole snake block,
          * and also large enough to compress data that actually grows bigger during compression. */
-        char *buf = g_malloc(MAX(VTE_SNAKE_BLOCKSIZE,
-                                 VTE_BLOCK_DATALENGTH_SIZE + VTE_OVERWRITE_COUNTER_SIZE + boa->compressBound));
+        char *buf = g_newa(char, MAX(VTE_SNAKE_BLOCKSIZE,
+                                     VTE_BLOCK_DATALENGTH_SIZE + VTE_OVERWRITE_COUNTER_SIZE + boa->compressBound));
 
         g_assert_cmpuint (offset, >=, boa->tail);
         g_assert_cmpuint (offset, <=, boa->head);
@@ -956,7 +959,7 @@ _vte_boa_write (VteBoa *boa, gsize offset, const char *data)
                         _vte_snake_write (&boa->parent, OFFSET_BOA_TO_SNAKE(offset), buf, VTE_SNAKE_BLOCKSIZE);
                         /* Try to punch out from the FS */
                         _vte_snake_write (&boa->parent, OFFSET_BOA_TO_SNAKE(offset), "", 0);
-                        goto out;
+                        return;
                 }
                 overwrite_counter++;
         }
@@ -981,9 +984,6 @@ _vte_boa_write (VteBoa *boa, gsize offset, const char *data)
         if (G_LIKELY (offset == boa->head)) {
                 boa->head += VTE_BOA_BLOCKSIZE;
         }
-
-out:
-        g_free(buf);
 }
 
 static void
@@ -1064,11 +1064,11 @@ _vte_file_stream_new (void)
 static void
 _vte_file_stream_init (VteFileStream *stream)
 {
-        stream->boa = g_object_new (VTE_TYPE_BOA, NULL);
+        stream->boa = (VteBoa *)g_object_new (VTE_TYPE_BOA, NULL);
         _vte_boa_init (stream->boa);
 
-        stream->rbuf = g_malloc(VTE_BOA_BLOCKSIZE);
-        stream->wbuf = g_malloc(VTE_BOA_BLOCKSIZE);
+        stream->rbuf = (char *)g_malloc(VTE_BOA_BLOCKSIZE);
+        stream->wbuf = (char *)g_malloc(VTE_BOA_BLOCKSIZE);
         stream->rbuf_offset = 1;  /* Invalidate */
 }
 
@@ -1241,6 +1241,8 @@ _vte_file_stream_class_init (VteFileStreamClass *klass)
 	klass->head = _vte_file_stream_head;
 }
 
+G_END_DECLS
+
 /******************************************************************************************/
 
 #ifdef VTESTREAM_MAIN
@@ -1299,7 +1301,7 @@ static void
 test_fakes (void)
 {
         char buf[100], buf2[100];
-        VteBoa *boa = g_object_new (VTE_TYPE_BOA, NULL);
+        VteBoa *boa = (VteBoa *)g_object_new (VTE_TYPE_BOA, NULL);
 
         /* Encrypt */
         strcpy(buf, "abcdXYZ1234!!!");
@@ -1361,7 +1363,7 @@ test_fakes (void)
 static void
 test_snake (void)
 {
-        VteSnake *snake = g_object_new (VTE_TYPE_SNAKE, NULL);
+        VteSnake *snake = (VteSnake *)g_object_new (VTE_TYPE_SNAKE, NULL);
 
         /* Test overwriting data */
         snake_write (snake, 0, "Armadillo");
@@ -1389,7 +1391,7 @@ test_snake (void)
 
         /* Start over */
         g_object_unref (snake);
-        snake = g_object_new (VTE_TYPE_SNAKE, NULL);
+        snake = (VteSnake *)g_object_new (VTE_TYPE_SNAKE, NULL);
 
         /* State 1 */
         snake_write (snake, 0, "Armadillo");
@@ -1464,7 +1466,7 @@ test_snake (void)
 static void
 test_boa (void)
 {
-        VteBoa *boa = g_object_new (VTE_TYPE_BOA, NULL);
+        VteBoa *boa = (VteBoa *)g_object_new (VTE_TYPE_BOA, NULL);
         VteSnake *snake = (VteSnake *) &boa->parent;
 
         /* State 1 */
